import socket
import threading

class SimpleServer:
    def __init__(self, port, max_clients = 5):
        # 初始化服务器套接字，并绑定指定的端口
        self.server_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.server_socket.bind(("", port))
        self.server_socket.listen(max_clients)
        
        self.clients = []  # 保存所有客户端连接的列表
        self.max_clients = max_clients  # 设置最大客户端连接数

    def start(self):
        print(f"Server started and listening on port ...")
        while True:
            # 等待新的客户端连接
            client_socket, client_address = self.server_socket.accept()
            print(f"Client {client_address} connected.")
            
            # 如果当前客户端数达到最大限制，则移除最早的客户端连接
            if len(self.clients) >= self.max_clients:
                old_client = self.clients.pop(0)
                old_client.close()
                print(f"Removed oldest client due to client limit reached.")

            # 为每个客户端连接创建一个新线程来处理它
            client_thread = threading.Thread(target=self.handle_client, args=(client_socket,))
            client_thread.daemon = True
            client_thread.start()
            self.clients.append(client_socket)

    def handle_client(self, client_socket):
        try:
            while True:
                # 从客户端接收数据
                data = client_socket.recv(1024)
                if not data:
                    break

                # 将接收到的数据广播给其他所有客户端
                
                self.broadcast(data, sender_socket=client_socket)

        except Exception as e:
            print(f"Error handling client: {e}")

        # 如果客户端断开连接或发生错误，则从客户端列表中移除
        self.remove_client(client_socket)

    def broadcast(self, data, sender_socket):
        # 遍历所有客户端，并发送数据，不包括发送数据的客户端本身
        for client in self.clients:
            if client != sender_socket:
                try:
                    client.send(data)
                except:
                    # 如果数据发送失败，例如因为客户端断开连接，则从客户端列表中移除
                    self.remove_client(client)

    def remove_client(self, client_socket):
        # 从客户端列表中移除指定的客户端，并关闭其连接
        if client_socket in self.clients:
            self.clients.remove(client_socket)
            client_socket.close()

if __name__ == "__main__":


    server = SimpleServer(5566)
    server_thread = threading.Thread(target=server.start)
    server_thread.start()


